/* snsrspfn.c

Functions to load sensor response functions into memory (in dynamic vectors)
from a file and to integrate functions of two variables (first is wavenumber,
second is temperature) with the sensor response function

    Copyright (C) 1999, 2000, 2001 Conrado Badenas <Conrado.Badenas@uv.es>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "snsrspfn.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "number.h"
#include "dvector.h"
#include "macros.h"
#include "planck.h"

#define MAXTYPES 2
void load_srf(FILE *file,dVector *nu,dVector *srf)
{
	char srftypes[1+MAXTYPES][8]={"unknown","cm-1","micro-m"};
	char cons[4][4]={"c1=","e1=","c2=","e2="};
	char line[256],*begincons;
	int hold;
	size_t type,i,j,n;
	Number c1,C1,erc1,erC1,c2,C2,erc2,erC2,nui,nuj,srfi,srfj,nu0,nu1;

	i_planck(0.,0.,0.,0.,0);g_planck(&C1,&erC1,&C2,&erC2,&hold);
	c1=C1;erc1=erC1;c2=C2;erc2=erC2;fgets(line,256,file);
	if ((begincons=strstr(line,cons[0]))!=NULL)
		if (sscanf(begincons+3,SN,&  c1)!=1)   c1=  C1;
	if ((begincons=strstr(line,cons[1]))!=NULL)
		if (sscanf(begincons+3,SN,&erc1)!=1) erc1=erC1;
	if ((begincons=strstr(line,cons[2]))!=NULL)
		if (sscanf(begincons+3,SN,&  c2)!=1)   c2=  C2;
	if ((begincons=strstr(line,cons[3]))!=NULL)
		if (sscanf(begincons+3,SN,&erc2)!=1) erc2=erC2;
	i_planck(c1,erc1,c2,erc2,1);
	sscanf(line,"%7s",srftypes[0]);for (type=1;type<=MAXTYPES;type++)
		if (!strcmp(srftypes[0],srftypes[type])) break;
	if (type>MAXTYPES)
		FATAL2("load_srf(): Type `%s' unsupported\n",srftypes[0])
	i_dvector(nu,0);i_dvector(srf,0);i=0;while (1)
	{
		if (fscanf(file,SN SN,&nui,&srfi)!=2) break;
		s_dvector(nu,i,nui);s_dvector(srf,i,srfi);
		i++;
	}
	if (i<3) FATAL2("load_srf(): SRF contains only %d response(s)!\n",i)
	n=nu->n;switch (type)
	{
		case 1:	break; /* Do nothing for cm-1 */
		case 2:	for (i=0;i<n;i++)
				s_dvector(nu,i,10000./g_dvector(nu,i));
			break; /* cm-1 = 10000 / micro-m */
		default:	FATAL("load_srf(): Never to reach here\n")
	}
	if (g_dvector(nu,0)>=g_dvector(nu,n-1)) for (i=0;i<n/2;i++)
	{
		j=n-1-i;
		nui =g_dvector(nu, i);nuj =g_dvector(nu, j);
		srfi=g_dvector(srf,i);srfj=g_dvector(srf,j);
		s_dvector(nu ,i,nuj );s_dvector(nu, j, nui);
		s_dvector(srf,i,srfj);s_dvector(srf,j,srfi);
	}
	for (nu0=g_dvector(nu,0),i=1;i<n;i++)
	{
		nu1=g_dvector(nu,i);if (nu1<=nu0)
			FATAL("load_srf(): SRF not strictly sorted\n")
		nu0=nu1;
	}
	if (g_dvector(srf,0) || !g_dvector(srf,1) || !g_dvector(srf,n-2)
		|| g_dvector(srf,n-1)) FATAL("load_srf(): SRF is not zero for "
			"first or last data, or is zero for second or "
			"penultimate data")
}
#undef MAXTYPES

Number intgrt_srf(Number (*f)(Number,Number),Number t,dVector *nu,dVector *srf)
{
	Number nu_im1,nu_i,f_i,f_im1,sum;
	size_t i;

	nu_im1=g_dvector(nu,0);f_im1=((*f)(nu_im1,t))*g_dvector(srf,0);
	sum=0.;for (i=1;i<nu->n;i++)
	{
		nu_i=g_dvector(nu,i);f_i=((*f)(nu_i,t))*g_dvector(srf,i);
		sum+=(f_i+f_im1)*(nu_i-nu_im1);nu_im1=nu_i;f_im1=f_i;
	}
	return (sum*0.5);
}
